/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.gui;

import java.io.File;
import java.util.Arrays;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ResultsMenuListener implements ChangeListener {

    private JMenu menu;
    private JFrame mainFrame;

    public ResultsMenuListener(JMenu m, JFrame f) {
        menu = m;
        mainFrame = f;
    }

    @Override
    public void stateChanged(ChangeEvent e) {
        menu.removeAll();
        File dir = new File(Consts.RES_PATH);
        if (!dir.exists()) {
            dir.mkdir();
        }
        String[] children = dir.list();
        Arrays.sort(children);
        if (children == null) {
            System.err
                    .println("Either dir does not exist or is not a directory");
        } else if (children.length == 0) {
            menu.add(new JMenuItem("No results available"));
        } else {
            for (String child : children) {
                JMenu submenu = new JMenu(child);
                menu.add(submenu);
                File subdir = new File(Consts.RES_PATH + File.separator + child);
                String[] grandchildren = subdir.list();
                Arrays.sort(grandchildren);
                for (String grandchild : grandchildren) {
                    JMenuItem resultItem = new JMenuItem(grandchild);
                    resultItem.addActionListener(new ResultsActionListener(
                            subdir + File.separator + grandchild, child,
                            grandchild, mainFrame));
                    submenu.add(resultItem);
                }
            }
        }
    }

}
