/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
class SettingsDialog extends JPanel {
	private static final long serialVersionUID = 1L;
	private static final int FIELD_COLUMNS = 20;

	private JTextField epmsField = new JTextField("", FIELD_COLUMNS);
	private JTextField dasField = new JTextField("", FIELD_COLUMNS);
	private JTextField taasField = new JTextField("", FIELD_COLUMNS);
	private JButton okButton;
	private boolean ok;
	private JDialog dialog;
	private Settings settings = new Settings(epmsField, dasField, taasField);

	public SettingsDialog() {
		setLayout(new BorderLayout());

		// construct a panel with user name and password fields

		JPanel panel = new JPanel();

		JPanel row1 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		JPanel row2 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		JPanel row3 = new JPanel(new FlowLayout(FlowLayout.RIGHT));

		panel.setLayout(new GridLayout(3, 1));
		row1.add(new JLabel("EPMS:", SwingConstants.RIGHT));
		row1.add(epmsField);

		row2.add(new JLabel("DAS:", SwingConstants.RIGHT));
		row2.add(dasField);

		row3.add(new JLabel("TAAS:", SwingConstants.RIGHT));
		row3.add(taasField);

		panel.add(row1);
		panel.add(row2);
		panel.add(row3);
		add(panel);

		// create Ok and Cancel buttons that terminate the dialog

		okButton = new JButton("Ok");
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				ok = true;
				settings.saveToFile();
				dialog.setVisible(false);
			}
		});

		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				dialog.setVisible(false);
			}
		});

		// add buttons to southern border

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		add(buttonPanel, BorderLayout.SOUTH);

	}

	/**
	 * Gets the dialog entries.
	 * 
	 * @return a User object whose state represents the dialog entries
	 */
	public Settings getSettings() {
		return settings;
	}

	/**
	 * Show the chooser panel in a dialog
	 * 
	 * @param parent
	 *            a component in the owner frame or null
	 * @param title
	 *            the dialog window title
	 */
	public boolean showDialog(Component parent, String title) {
		ok = false;

		// locate the owner frame

		Frame owner = null;
		if (parent instanceof Frame) {
			owner = (Frame) parent;
		} else {
			owner = (Frame) SwingUtilities.getAncestorOfClass(Frame.class,
					parent);
		}

		// if first time, or if owner has changed, make new dialog

		if (dialog == null || dialog.getOwner() != owner) {
			dialog = new JDialog(owner, true);
			dialog.add(this);
			dialog.getRootPane().setDefaultButton(okButton);
			dialog.setLocationRelativeTo(dialog.getOwner());
			dialog.pack();
			dialog.setResizable(false);
		}

		// set title and show dialog

		dialog.setTitle(title);
		dialog.setVisible(true);
		return ok;
	}

}
