/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.gui;

import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import kgrid.client.workflow.DatasetNode;
import kgrid.client.workflow.ToolNode;
import kgrid.client.workflow.ModelNode;
import kgrid.client.workflow.Workflow;
import kgrid.client.workflow.WorkflowNode;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class TableMouseListener extends MouseAdapter {

    private Workflow        workflow;
    private PropertiesPanel panel;
    private WorkflowNode    node;

    public TableMouseListener(Workflow w, PropertiesPanel p) {
        workflow = w;
        panel = p;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        if (SwingUtilities.isRightMouseButton(e)) {
            Object obj = workflow.getSelectedCell();
            if (obj instanceof WorkflowNode) {
                node = (WorkflowNode) obj;
                JPopupMenu popup = new JPopupMenu();
                JMenuItem item = null;
                JMenuItem item2 = null;

                if (node instanceof DatasetNode) {
                    item = new JMenuItem(addAttrAction);
                } else if (node instanceof ToolNode) {
                    // TODO KGClient: get focused row, not selected one
                    if (panel.getArgTable().getSelectedRow() != -1) {
                        item2 = new JMenuItem(delArgAction);
                    }
                }
                if (item != null) {
                    popup.add(item);
                }
                if (item2 != null) {
                    popup.add(item2);
                }
                if (item != null || item2 != null) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        } else if (SwingUtilities.isLeftMouseButton(e)) {
            Object obj = workflow.getSelectedCell();
            if (obj instanceof WorkflowNode) {
                node = (WorkflowNode) obj;
                if (node instanceof ModelNode) {
                    int row = panel.getSrcPrmTable().getSelectedRow();
                    if (row > 0) {
                        String dir = (String) panel.getSrcPrmTable().getModel()
                                .getValueAt(row, 0);
                        String fileName = (String) panel.getSrcPrmTable()
                                .getModel().getValueAt(row, 1);
                        String filePath = Consts.RES_PATH + File.separator
                                + dir + File.separator + fileName;
                        if (new File(filePath).exists()) {
                            ResultsActionListener ral = new ResultsActionListener(
                                    filePath, dir, fileName, null);
                            ral.actionPerformed(null);
                        } else {
                            JOptionPane.showMessageDialog(panel, "File '" + dir
                                    + File.separator + fileName
                                    + "' does not exist.");
                        }
                    }
                }
            }
        }
    }

    private Action addAttrAction = new AbstractAction("Add Attribute") {
                                     private static final long serialVersionUID = 1L;

                                     public void actionPerformed(
                                             ActionEvent event) {
                                         ((DatasetNode) node).addAttribute();
                                         panel.changeTable(node);
                                     }
                                 };

    private Action delArgAction  = new AbstractAction(
                                         "Delete selected argument") {
                                     private static final long serialVersionUID = 1L;

                                     public void actionPerformed(
                                             ActionEvent event) {
                                         String argument = (String) panel
                                                 .getArgTable()
                                                 .getModel()
                                                 .getValueAt(
                                                         panel
                                                                 .getArgTable()
                                                                 .getSelectedRow(),
                                                         0);
                                         if (JOptionPane.showConfirmDialog(
                                                 panel,
                                                 "Are you sure you want to delete '"
                                                         + argument
                                                         + "' argument?",
                                                 "Delete confirmation",
                                                 JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                                             ((ToolNode) node)
                                                     .delArgument(argument);
                                             panel.updateArgTable(node);
                                         }
                                     }
                                 };
}
