/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.remote;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.cert.X509Certificate;
import java.util.List;

import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import kgrid.client.gui.Settings;
import kgrid.client.workflow.Workflow;
import kgrid.epms.stubs.EPMSFactory.CreateResource;
import kgrid.epms.stubs.EPMSFactory.CreateResourceResponse;
import kgrid.epms.stubs.EPMSFactory.EPMSFactoryPortType;
import kgrid.epms.stubs.EPMSFactory.service.EPMSFactoryServiceAddressingLocator;
import kgrid.epms.stubs.EPMSService.EPMSPortType;
import kgrid.epms.stubs.EPMSService.ExecutionParameters;
import kgrid.epms.stubs.EPMSService.Parameters;
import kgrid.epms.stubs.EPMSService.service.EPMSServiceAddressingLocator;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.globus.axis.util.Util;
import org.globus.delegation.DelegationUtil;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.core.notification.ResourcePropertyValueChangeNotificationElementType;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.utils.AddressingUtils;
import org.oasis.wsn.Subscribe;
import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.properties.ResourcePropertyValueChangeNotificationType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class EPMSTask implements NotifyCallback, Consts {

	private Parameters param;
	private EPMSPortType portType;
	private String factoryURI;
	private Workflow workflow;
	private EndpointReferenceType endPointReference;

	static {
		Util.registerTransport();
	}

	public static void printDocument(Document doc) {
		try {
			PrintWriter pw = new PrintWriter(System.out);
			DOMSerializer ser = new XMLSerializer(pw, new OutputFormat(doc,
					"UTF-8", true));
			ser.serialize(doc);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void invoke() throws Exception {
		kgrid.client.gui.Utils.print("Invoking the EPMS service");
		Object obj;
		obj = portType.submitKApplication(param);
		if (obj == null) {
			String message = "The EPMS has returned a null object";
			kgrid.client.gui.Utils.print(message);
			throw new Exception(message);
		} else if (obj instanceof ExecutionParameters) {
			ExecutionParameters ris = (ExecutionParameters) obj;
			kgrid.client.gui.Utils
					.print("The result is an ExecutionResults object");
			Object obj2 = ris.getExecutionPlanDoc();
			if (obj2 instanceof Document) {
				kgrid.client.gui.Utils
						.print("Obtaining a Document object from the ExecutionResults object");
				// printDocument(executionPlanDoc);
			} else {
				kgrid.client.gui.Utils.print("Obtaining a "
						+ obj2.getClass().getName()
						+ " object from the ExecutionResults object");
			}
		} else {
			kgrid.client.gui.Utils.print("The result is a "
					+ obj.getClass().getName() + " object");
		}
	}

	public EPMSTask(Workflow w) throws Exception {
		workflow = w;

		EPMSFactoryServiceAddressingLocator factoryLocator = new EPMSFactoryServiceAddressingLocator();
		EPMSServiceAddressingLocator instanceLocator = new EPMSServiceAddressingLocator();

		factoryURI = "https://" + Settings.getEPMSAddress() + ":"
				+ Settings.getEPMSPort() + "/" + EPMS_FACTORY;

		EndpointReferenceType factoryEPR, instanceEPR;
		EPMSFactoryPortType epmsFactory;

		// 1: get Factory Port Type
		factoryEPR = new EndpointReferenceType();
		factoryEPR.setAddress(new Address(factoryURI));
		epmsFactory = factoryLocator.getEPMSFactoryPortTypePort(factoryEPR);

		// 2: Create resource and get endpoint reference of WS-Resource.
		// This resource is our "instance".
		kgrid.client.gui.Utils
				.print("Creating resource and getting endpoint reference of WS-Resource");
		CreateResourceResponse createResponse = epmsFactory
				.createResource(new CreateResource());
		instanceEPR = createResponse.getEndpointReference();

		// 3: ?
		portType = instanceLocator.getEPMSPortTypePort(instanceEPR);
		subscribeToEPMSModelTopic(portType);

		endPointReference = delegateCredential();

		String fileName = workflow.saveToTempFile();

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = dbf.newDocumentBuilder();
		Document document = builder.parse(new File(fileName));

		param = new Parameters(document, endPointReference);

	}

	// Contact the Delegation Factory Service and get an EPR
	// for the Delegation Resource that contains your delegated credential.
	private static EndpointReferenceType delegateCredential() throws Exception {
		String host = Settings.getEPMSAddress();
		int port = Settings.getEPMSPort();

		ClientSecurityDescriptor desc = getClientSecDesc();
		GlobusCredential credential = GlobusCredential.getDefaultCredential();
		String factoryUrl = ("https://" + host + ":" + port + "/" + DELEGATION_FACTORY);
		System.out.println("factoryUrl = " + factoryUrl);
		int terminationTime = 60;
		int lifetime = terminationTime * 60; // lifetime in seconds
		EndpointReferenceType delegEpr = AddressingUtils
				.createEndpointReference(factoryUrl, null);
		X509Certificate[] certsToDelegateOn = DelegationUtil
				.getCertificateChainRP(delegEpr, desc);
		X509Certificate certToSign = certsToDelegateOn[0];
		return DelegationUtil.delegate(factoryUrl, credential, certToSign,
				lifetime, false, desc);
	}

	private static ClientSecurityDescriptor getClientSecDesc() {
		ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
		Authorization authzVal = HostAuthorization.getInstance();
		Object authVal = GSIConstants.SIGNATURE;
		desc.setGSITransport((Integer) authVal);
		Util.registerTransport();
		desc.setAuthz(authzVal);
		return desc;
	}

	/**
	 * The deliver function of the interface {@link NotifyCallback} will be
	 * invoked whenever a notification for that consumer arrives. The message
	 * parameter will either be of {@link Element} type if the message type was
	 * unknown or some object mapped to the type of the message.
	 */
	public synchronized void deliver(List topicPath,
			EndpointReferenceType producer, Object message) {
		synchronized (this) {
			ResourcePropertyValueChangeNotificationElementType notifElem;
			ResourcePropertyValueChangeNotificationType notif;

			notifElem = (ResourcePropertyValueChangeNotificationElementType) message;
			notif = notifElem.getResourcePropertyValueChangeNotification();
			if (notif != null) {
				for (MessageElement me : notif.getNewValue().get_any()) {
					String notifMessage = me.getValue();
					if (notifMessage.startsWith("#STATE")) {
						String id = notifMessage.substring(
								notifMessage.indexOf(":") + 1,
								notifMessage.indexOf("$"));
						String state = notifMessage.substring(notifMessage
								.indexOf("$") + 1);
						workflow.update(id, state);
						if (state.startsWith("FAILED")) {
							Utils.showDialog(
									"Failed operation on "
											+ workflow.getNodeType(id)
											+ " node " + id, state,
									JOptionPane.ERROR_MESSAGE);
						} else if (state.startsWith("ERROR")) {
							Utils.showDialog(
									"Error on " + workflow.getNodeType(id)
											+ " node " + id, state,
									JOptionPane.ERROR_MESSAGE);
						}
					} else if (notifMessage.startsWith("#ERROR")) {
						Utils.showDialog("Error", notifMessage.substring(7),
								JOptionPane.ERROR_MESSAGE);
					}
					Utils.print(notifMessage, producer.getAddress());
				}
			}
			this.notify();
		}
	}

	/**
	 * Subscribe the specified listener as consumer of notification 'KDSModel'
	 * topic of the specified KDS Service
	 * 
	 * @param kds
	 *            KDSPortType of the KDS Service whose the topic is a resource
	 *            property
	 */
	public void subscribeToEPMSModelTopic(EPMSPortType epmsPT) throws Exception {

		// 1: set up the notification consumer
		// (the listener will be set up as a notification consumer)
		NotificationConsumerManager consumer = null;

		// 2: set the 'listener' as a 'notification consumer'
		consumer = NotificationConsumerManager.getInstance();
		consumer.startListening();
		EndpointReferenceType consumerEPR = consumer
				.createNotificationConsumer(this);

		// 3: create (and opportunetely set) the remote subscribe request
		// Create the request to the remote Subscribe() call
		Subscribe request = new Subscribe();
		// Must the notification be delivered using the Notify
		// operation?
		request.setUseNotify(Boolean.TRUE);
		// Indicate what the consumer client's EPR is
		request.setConsumerReference(consumerEPR);

		// 4: create the topic to be subscribed to
		// The TopicExpression specifies what topic we want to
		// subscribe to
		TopicExpressionType topicExpression = new TopicExpressionType();
		topicExpression.setDialect(WSNConstants.SIMPLE_TOPIC_DIALECT);
		topicExpression.setValue(EPMS_QNAME);
		request.setTopicExpression(topicExpression);

		// 6: send the subscribe request to the notification producer (EPMS)

		epmsPT.subscribe(request);
	}

	public EndpointReferenceType getEPR() {
		return endPointReference;
	}

}
