/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.remote;

import java.io.ByteArrayInputStream;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import kgrid.client.gui.Settings;
import kgrid.client.gui.View;
import kgrid.raems.stubs.RAEMSFactory.RAEMSFactoryPortType;
import kgrid.raems.stubs.RAEMSFactory.service.RAEMSFactoryServiceAddressingLocator;
import kgrid.raems.stubs.RAEMSService.RAEMSPortType;
import kgrid.raems.stubs.RAEMSService.service.RAEMSServiceAddressingLocator;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.AttributedURI;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.w3c.dom.Document;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Utils implements Consts {

	private static final String DATE_PATTERN = "yyyy.MM.dd HH:mm:ss";
	private static final SimpleDateFormat DATE_FORMATTER = new SimpleDateFormat(
			DATE_PATTERN);

	public static void print(String s, AttributedURI address) {
		String path = address.getPath();

		String dateAndTime = DATE_FORMATTER.format(new Date());
		System.out.println(dateAndTime
				+ " "
				+ path.substring(path.lastIndexOf('/') + 1).replace("Service",
						"") + "@" + address.getHost() + " " + s);
	}

	public static Document xmlToDocument(String xml) throws Exception {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

		factory.setNamespaceAware(true);
		DocumentBuilder builder = factory.newDocumentBuilder();

		return builder.parse(new ByteArrayInputStream(xml.getBytes()));
	}

	/**
	 * Initialize RAEMS Resource and get the RAEMS Instance Service Port Type.
	 * 
	 * @throws RemoteException
	 */
	public static kgrid.raems.stubs.RAEMSService.ArrayOfString getRAEMSQueryAOS(
			String xml) throws RemoteException {

		RAEMSPortType raemsPT = null;
		try {
			RAEMSFactoryServiceAddressingLocator factoryLocator = new RAEMSFactoryServiceAddressingLocator();
			RAEMSServiceAddressingLocator instanceLocator = new RAEMSServiceAddressingLocator();

			String raemsFactoryURI = "https://" + Settings.getEPMSAddress()
					+ ":" + Settings.getEPMSPort() + "/" + RAEMS_FACTORY;

			EndpointReferenceType factoryEPR, instanceEPR;
			RAEMSFactoryPortType raemsFactory;

			factoryEPR = new EndpointReferenceType();
			factoryEPR.setAddress(new Address(raemsFactoryURI));
			raemsFactory = factoryLocator
					.getRAEMSFactoryPortTypePort(factoryEPR);

			kgrid.raems.stubs.RAEMSFactory.CreateResourceResponse createResponse = raemsFactory
					.createResource(new kgrid.raems.stubs.RAEMSFactory.CreateResource());
			instanceEPR = createResponse.getEndpointReference();

			raemsPT = instanceLocator.getRAEMSPortTypePort(instanceEPR);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return raemsPT.getQueryFromXML(xml);
	}

	public static void showDialog(String title, String message, int type) {
		class MyThread extends Thread {
			private String message;
			private String title;
			private int type;

			public MyThread(String t, String m, int ty) {
				title = t;
				message = m;
				type = ty;
			}

			@Override
			public void run() {
				// TODO display the full error message
				JOptionPane.showMessageDialog(View.getMainFrame(), message,
						title, type);
			}
		}
		new MyThread(title, message, type).start();

	}

}
