/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.workflow;

import org.jdom.Element;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Edge extends DefaultEdge implements Consts {

	private static final long serialVersionUID = 1L;
	private static int arrow = GraphConstants.ARROW_CLASSIC;
	private int id;

	public static final String[][] LABELS_MATRIX = { { TRANSFER, "?", null },
			{ DATASET, null, MODEL }, { null, MODEL, TRANSFER } };

	private String type = "";
	private int index = 0;
	private int sourceType;
	private int targetType;
	private boolean labelEditable;
	private WorkflowNode sourceNode;
	private WorkflowNode targetNode;

	public boolean isFromToolToDataset() {
		return sourceType == TOOL_TYPE && targetType == DATASET_TYPE;
	}

	/**
	 * @return the sourceNode
	 */
	public WorkflowNode getSourceNode() {
		return sourceNode;
	}

	/**
	 * @param sourceNode
	 *            the sourceNode to set
	 */
	public void setSourceNode(WorkflowNode sourceNode) {
		this.sourceNode = sourceNode;
	}

	/**
	 * @return the targetNode
	 */
	public WorkflowNode getTargetNode() {
		return targetNode;
	}

	/**
	 * @param targetNode
	 *            the targetNode to set
	 */
	public void setTargetNode(WorkflowNode targetNode) {
		this.targetNode = targetNode;
	}

	public Edge(int id) {
		this.id = id;
		// super(new String("train / test"));
		GraphConstants.setLineEnd(getAttributes(), arrow);
		GraphConstants.setEndFill(getAttributes(), true);
		GraphConstants.setLabelAlongEdge(getAttributes(), true);
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getType() {
		return type;
	}

	@Override
	public String toString() {
		if (index == 0) {
			return type;
		}
		return type + index;
	}

	public void setSourceAndTargetTypes(int type1, int type2) {
		sourceType = type1;
		targetType = type2;
		labelEditable = sourceType == DATASET_TYPE && targetType == TOOL_TYPE;
	}

	public boolean isLabelEditable() {
		return labelEditable;
	}

	public GraphElement getElement() {
		GraphElement edgeE = new GraphElement("edge", "e" + id);

		edgeE.setAttribute("id", "e" + id);

		String source = ((WorkflowNode) ((DefaultPort) getSource()).getRoot())
				.getIdName();
		String target = ((WorkflowNode) ((DefaultPort) getTarget()).getRoot())
				.getIdName();

		edgeE.setAttribute("source", source);
		edgeE.setAttribute("target", target);

		Element labelE = new Element("data");
		labelE.setAttribute("key", "type");
		labelE.setText(type);
		edgeE.addContent(labelE);

		return edgeE;
	}

	public void setIndex(int i) {
		index = i;
	}

}
