/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.workflow;

import java.util.Date;

import org.apache.commons.lang3.time.DurationFormatUtils;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Timer {

	private long start;
	private long duration;

	public void start() {
		start = System.currentTimeMillis();
	}

	public void stop() {
		duration = System.currentTimeMillis() - start;
	}

	public long getStartingTime() {
		return start;
	}

	public long getDuration() {
		return duration;
	}

	public static String showTimeFromMilliSec(long milliSeconds) {

		long seconds = milliSeconds / 1000;
		long hours = seconds / 3600;
		long minutes = seconds / 60 - hours * 60;
		long sec = seconds - (hours * 3600 + minutes * 60);

		String outputString;

		if (hours != 0) {
			outputString = hours + " hours " + minutes + " minutes and " + sec
					+ " seconds (" + seconds + " seconds)";
		} else if (minutes != 0) {
			outputString = minutes + " minutes " + sec + " seconds (" + seconds
					+ " seconds)";
		} else {
			outputString = sec + " seconds";
		}

		return outputString;
	}

	public static void main(String[] args) {
		System.out.println("formatDurationHMS = "
				+ DurationFormatUtils.formatDurationHMS(new Date().getTime()));
		System.out.println("formatDurationISO = "
				+ DurationFormatUtils.formatDurationISO(new Date().getTime()));
		System.out.println("formatDurationWords = "
				+ DurationFormatUtils.formatDurationWords(new Date().getTime(),
						true, true));
	}

}
